;Startup v1.0 by Brandon Wilson
;
;Startup is a program that allows you to choose what you want
; to run when your calculator is first turned on.
;You have the option to either display a picture with a delay or until
; a key is pressed, display a menu, or run a Basic or assembly program.
;
;Notes: Basic program support is kind of choppy. Try to use only simple
;        programs with it.
;Bugs:  Basic command Output includes quotation marks
;       Basic homescreen text may not be in the right place if calc was
;        turned off with non-empty line, which is common with APD
;       Basic command InpSt will cause memory error

.nolist
#include "ti86.inc"
.list

sFlags          equ     asm_flag1
on_enabled      equ     0

menuPtr         equ     _plotSScreen

.org _asm_exec_ram
        ld hl,_errExitProgram           ;install error handler in case
        call _pushErrorHandler          ; anything goes wrong

        call _flushAllMenus             ;write top part of screen
        call _runIndicOff               ;
        res on_enabled,(iy+sFlags)      ;
        call _clrLCD                    ;
        ld hl,0*256+38                  ;
        ld (_penCol),hl                 ;
        ld hl,sTitle                    ;
        call _vputs                     ;
        ld hl,0FC00h                    ;invert it
        ld b,16*7                       ;
xorLoop:ld a,(hl)                       ;
        xor 0FFh                        ;
        ld (hl),a                       ;
        inc hl                          ;
        djnz xorLoop                    ;
        call clearArea                  ;clear middle area

mainMenu:
        ld hl,bDummyMenu                ;display main menu and choices
        call _ASAPloadmenu              ;
        call _funcDisp                  ;
        ld hl,58*256+3                  ;
        ld (_pencol),hl                 ;
        ld hl,sMain1                    ;
        call _vputs                     ;
        bit alt_on,(iy+exceptionflg)    ;on routine already installed?
        jr z,_skipUninst                ;
        ld hl,_alt_on_exec+2            ;see if installed routine is ours
        call _ldhlind                   ;
        ld de,$6259                     ;
        call _cphlde                    ;compare signature bytes
        jr nz,_skipUninst               ;
        set on_enabled,(iy+sFlags)      ;make uninstall a choice too
        ld hl,58*256+29                 ;
        ld (_penCol),hl                 ;
        ld hl,sMain2                    ;
        call _vputs                     ;
_skipUninst:                            ;
        ld hl,58*256+107                ;display rest of choices
        ld (_penCol),hl                 ;
        ld hl,sMain5                    ;
        call _vputs                     ;
        ld hl,10*256+5                  ;
        ld (_penCol),hl                 ;
        ld hl,sMainMessage1             ;
        call _vputs                     ;
        ld hl,18*256+10                 ;
        ld (_penCol),hl                 ;
        ld hl,sMainMessage2             ;
        call _vputs                     ;
        ld hl,28*256+5                  ;
        ld (_penCol),hl                 ;
        ld hl,sMainMessage3             ;
        call _vputs                     ;
gk1Loop:call _getCSC                    ;main menu key loop
        or a                            ;
        jr z,gk1Loop                    ;
        cp K_F5                         ;
        jp z,exitProgram                ;
        cp K_F1                         ;
        jr z,_gkF1                      ;
        bit on_enabled,(iy+sFlags)      ;
        jr z,gk1Loop                    ;
        cp K_F2                         ;
        jr nz,gk1Loop                   ;
        res alt_on,(iy+exceptionflg)    ;uninstall and display status
        ld hl,42*256+15                 ;
        ld (_penCol),hl                 ;
        ld hl,sUninstalled              ;
        call _vputs                     ;
        jp mainMenu                     ;

_gkF1:  ld hl,bDummyMenu                ;display install menu
        call _ASAPloadmenu              ;
        call _funcDisp                  ;
        ld hl,58*256+5                  ;
        ld (_penCol),hl                 ;
        ld hl,sInstall1                 ;
        call _vputs                     ;
        ld hl,58*256+34                 ;
        ld (_penCol),hl                 ;
        ld hl,sInstall2                 ;
        call _vputs                     ;
        ld hl,58*256+54                 ;
        ld (_penCol),hl                 ;
        ld hl,sInstall3                 ;
        call _vputs                     ;
        ld hl,58*256+78                 ;
        ld (_penCol),hl                 ;
        ld hl,sInstall4                 ;
        call _vputs                     ;
        ld hl,58*256+106                ;
        ld (_penCol),hl                 ;
        ld hl,sInstall5                 ;
        call _vputs                     ;
        ld hl,0FC00h+(16*56)+9          ;phase out line at 'Install'
        ld de,16                        ;
        ld b,2                          ;
_eraseLoop1:                            ;
        ld a,(hl)                       ;
        and %11001111                   ;
        ld (hl),a                       ;
        add hl,de                       ;
        djnz _eraseLoop1                ;
        ld b,7                          ;
_eraseLoop2:                            ;
        ld a,(hl)                       ;
        and %10001111                   ;
        or %01000000                    ;
        ld (hl),a                       ;
        add hl,de                       ;
        djnz _eraseLoop2                ;
        call clearArea                  ;
        ld hl,30*256+14                 ;
        ld (_penCol),hl                 ;
        ld hl,sInstallMessage1          ;
        call _vputs                     ;
_gk2Loop:                               ;
        call _getCSC                    ;install menu key loop
        or a                            ;
        jr z,_gk2Loop                   ;
        cp K_F5                         ;
        jp z,_gk2F5                     ;
        cp K_F1                         ;
        jp z,_gk2F1                     ;
        cp K_F2                         ;
        jp z,_gk2F2                     ;
        cp K_F3                         ;
        jp z,_gk2F3                     ;
        cp K_LEFT                       ;
        jp z,_gk2Left                   ;
        cp K_RIGHT                      ;
        jp z,_gk2Right                  ;
        cp K_F4                         ;
        jr nz,_gk2Loop                  ;
        ld a,(bType)                    ;if picture, display second menu
        cp 02h                          ;
        jr nz,_contInstall              ;
        ld hl,bPicMenu                  ;display picture configure menu
        call _ASAPloadmenu              ;
        call _funcDisp                  ;
_gkPicLoop:                             ;picture configure menu key loop
        call _getCSC                    ;
        or a                            ;
        jr z,_gkPicLoop                 ;
        cp K_F1                         ;
        jr z,_picF1                     ;
        cp K_F2                         ;
        jr z,_picF2                     ;
        cp K_F3                         ;
        jr z,_picF3                     ;
        cp K_F5                         ;
        jr nz,_gkPicLoop                ;
        call clearArea                  ;
        jp _gkF1                        ;
_picF1: xor a                           ;no delay; wait for key
        ld (bPicDelay),a                ;
        jr _contInstall                 ;
_picF2: ld a,2                          ;2 second delay
        ld (bPicDelay),a                ;
        jr _contInstall                 ;
_picF3: ld a,5                          ;5 second delay
        ld (bPicDelay),a                ;
        jr _contInstall                 ;
_contInstall:                           ;
        ld hl,_op1                      ;enable user on routine
        ld de,bName                     ;
        ld bc,10                        ;
        ldir                            ;
        ld hl,_altOnRoutine             ;copy to alt_on area
        ld de,_alt_on_exec              ;
        ld bc,200                       ;
        ldir                            ;
        ld de,$28                       ;calculate checksum
        ld a,(_alt_on_exec)             ;
        ld hl,_alt_on_chksum+($28*1)    ;
        add a,(hl)                      ;
        ld hl,_alt_on_chksum+($28*2)    ;
        add a,(hl)                      ;
        ld hl,_alt_on_chksum+($28*3)    ;
        add a,(hl)                      ;
        ld hl,_alt_on_chksum+($28*4)    ;
        add a,(hl)                      ;
        ld hl,_alt_on_chksum+($28*5)    ;
        add a,(hl)                      ;
        ld (_alt_on_chksum),a           ;store the checksum
        set alt_on,(iy+$23)             ;enable user routine
exitProgram:
        call _popErrorHandler           ;
        call _clrLCD                    ;exit
        call _homeUp                    ;
        ret                             ;
        
_gk2F5: call clearArea                  ;go back to main menu
        jp mainMenu                     ;
_gk2F1: call clearArea                  ;display program section
        ld hl,12*256+5                  ;
        ld (_penCol),hl                 ;
        ld hl,sPrgmMessage1             ;
        call _vputs                     ;
        ld hl,bDummyPrgmVar             ;
        rst 20h                         ;
_refreshPrgm:                           ;update program section
        xor a                           ;
        call _FindAlphaUp               ;
        ld hl,0104h                     ;
        ld (_curRow),hl                 ;
        ld a,Lleft                      ;
        call _PutC                      ;
        call _EraseEOL                  ;
        ld hl,1304h                     ;
        ld (_curRow),hl                 ;
        ld a,Lconvert                   ;
        call _PutC                      ;
        rst 10h                         ;assembly or basic?
        call _ok_to_edit_prog           ;
        jr z,_isAsm                     ;
        xor a                           ;
        jr _gk2F1Cont                   ;
_isAsm: ld a,1                          ;
_gk2F1Cont:                             ;
        ld (bType),a                    ;store whether assembly or basic
        ld a,(_op1+1)                   ;get center screen position
        ld b,a                          ;
        ld a,21                         ;
        sub b                           ;
        sra a                           ;
        ld (_curCol),a                  ;
        ld hl,_op1+1                    ;
        call _putps                     ;display name
        jp _gk2Loop                     ;
_gk2F2: ld a,2                          ;display picture section
        ld (bType),a                    ;
        call clearArea                  ;
        ld hl,12*256+5                  ;
        ld (_penCol),hl                 ;
        ld hl,sPicMessage1              ;
        call _vputs                     ;
        ld hl,bDummyPicVar              ;
        rst 20h                         ;
_refreshPic:                            ;update picture section
        xor a                           ;
        call _FindAlphaUp               ;
        jr nc,_gkF2Found                ;
        ld a,(_op1+2)                   ;still dummy var (not found)?
        or a                            ;
        jr z,_noPicFound                ;
        cp '<'                          ;
        jp nz,_gk2Loop                  ;
_noPicFound:                            ;
        ld hl,sNone                     ;none found, make name <None>
        ld de,_op1                      ;
        ld bc,10                        ;
        ldir                            ;
_gkF2Found:                             ;get center screen position
        ld hl,0104h                     ;
        ld (_curRow),hl                 ;
        ld a,Lleft                      ;
        call _PutC                      ;
        call _EraseEOL                  ;
        ld hl,1304h                     ;
        ld (_curRow),hl                 ;
        ld a,Lconvert                   ;
        call _PutC                      ;
        ld a,(_op1+1)                   ;
        ld b,a                          ;
        ld a,21                         ;
        sub b                           ;
        sra a                           ;
        ld (_curCol),a                  ;
        ld hl,_op1+1                    ;
        call _putps                     ;display name
        jp _gk2Loop                     ;
_gk2F3: call clearArea                  ;display menu section
        ld hl,bMenuData                 ;
        ld (menuPtr),hl                 ;
        ld hl,12*256+7                  ;
        ld (_penCol),hl                 ;
        ld hl,sMenuMessage1             ;
        call _vputs                     ;
_refreshMenu:                           ;update menu section
        ld hl,0104h                     ;
        ld (_curRow),hl                 ;
        ld a,Lleft                      ;
        call _PutC                      ;
        call _EraseEOL                  ;
        ld hl,1304h                     ;
        ld (_curRow),hl                 ;
        ld a,Lconvert                   ;
        call _PutC                      ;
        ld hl,(menuPtr)                 ;
        rst 20h                         ;store current menu name
        ld a,(_op1)                     ;get center screen position
        ld (bType),a                    ;
        ld a,(_op1+1)                   ;
        ld b,a                          ;
        ld a,21                         ;
        sub b                           ;
        sra a                           ;
        ld (_curCol),a                  ;
        ld hl,_op1+1                    ;
        call _putps                     ;display current name
        jp _gk2Loop                     ;
_gk2Left:                               ;move left in current list
        ld a,(bType)                    ;
        or a                            ;what section are we in?
        jr z,_typePrgmL                 ;program?
        cp 1                            ;
        jr z,_typePrgmL                 ;
        cp 2                            ;
        jr z,_typePicL                  ;picture?
        ld hl,(menuPtr)                 ;menu
        ld de,bMenuData                 ;
        call _cphlde                    ;at first item?
        jp z,_gk2Loop                   ;
        ld bc,7                         ;move left
        or a                            ;
        sbc hl,bc                       ;
        ld (menuPtr),hl                 ;
        jp _refreshMenu                 ;
_typePrgmL:                             ;program
        xor a                           ;
        call _FindAlphaDn               ;go back one  
        jp c,_gk2Loop                   ;
        xor a                           ;again so update will find right one
        call _FindAlphaDn               ;
        jp nc,_refreshPrgm              ;refresh if we can
        ld hl,bDummyPrgmVar             ;can't, start over
        rst 20h                         ;
        jp _refreshPrgm                 ;
_typePicL:                              ;picture
        xor a                           ;
        call _FindAlphaDn               ;go back one
        jp c,_gk2Loop                   ;
        xor a                           ;again so update will find right one
        call _FindAlphaDn               ;
        jp nc,_refreshPic               ;refresh if we can
        ld hl,bDummyPicVar              ;can't, start over
        rst 20h                         ;
        jp _refreshPic                  ;
_gk2Right:                              ;move right in current list
        ld a,(bType)                    ;
        or a                            ;what section are we in?
        jp z,_refreshPrgm               ;program?
        cp 1                            ;
        jp z,_refreshPrgm               ;
        cp 2                            ;
        jp z,_refreshPic                ;picture?
        ld hl,(menuPtr)                 ;menu
        ld de,bMenuData+14              ;
        call _cphlde                    ;at last item?
        jp z,_gk2Loop                   ;
        ld de,7                         ;move left
        add hl,de                       ;
        ld (menuPtr),hl                 ;
        jp _refreshMenu                 ;

clearArea:
        ld hl,0FC00h+(16*9)             ;clear middle screen area
        ld bc,45*16                     ;
_caLoop:ld (hl),0                       ;
        inc hl                          ;
        dec bc                          ;
        ld a,b                          ;
        or c                            ;
        jr nz,_caLoop                   ;
        ret                             ;

sTitle: .db "Startup v1.0",0
sUninstalled:
        .db "Startup has been uninstalled!",0
sMainMessage1:
        .db "Choose Install to "
        .db "enable Startup",0
sMainMessage2:
        .db "or choose Uninst "
        .db "to disable it.",0
sMainMessage3:
        .db "Choose Exit to exit "
        .db "the program.",0
sInstallMessage1:
        .db "Choose what to run at startup.",0
sPrgmMessage1:
        .db "Select a program and choose "
        .db "Install:",0
sPicMessage1:
        .db "Select a picture and choose "
        .db "Install:",0
sMenuMessage1:
        .db "Select a menu and choose "
        .db "Install:",0
sMain1: .db "Install",0
sMain2: .db "Uninst",0
sMain5: .db "Exit",0
sInstall1:
        .db "Prgm",0
sInstall2:
        .db "Pic",0
sInstall3:
        .db "Menu",0
sInstall4:
        .db "Install",0
sInstall5:
        .db "Back",0
bDummyMenu:
        .db 09h
        .db 1
        .dw bDummyData
bDummyData:
        .db 0," ",0
bDummyPrgmVar:
        .db 12h,01h,0
bDummyPicVar:
        .db 11h,01h,0
bPicMenu:
        .db 05h
        .db 5
        .dw bPic1
        .dw bPic2
        .dw bPic3
        .dw bPic4
        .dw bPic5
bPic1:  .db 0,"Key",0
bPic2:  .db 0,"2 sec",0
bPic3:  .db 0,"5 sec",0
bPic4:  .db 0," ",0
bPic5:  .db 0,"Back",0
sNone:  .db 11,6,"<None>",0
bMenuData:
        .db 3,4,"MATH",0
        .db 4,4,"PRGM",0
        .db 5,3,"MEM",0

_altOnRoutine:
        jr _startOnRoutine
        .dw $6259                       ;signature word
_startOnRoutine:
        call _CursorOff                 ;turn off cursor
        ld a,(bType)                    ;go to right section
        or a                            ;
        jp z,_executeBasic-_altOnRoutine+_alt_on_exec
        cp 1                            ;
        jp z,_executeAssembly-_altOnRoutine+_alt_on_exec
        cp 2                            ;
        jr z,_displayPicture            ;
        cp 3                            ;
        jr z,_displayMathMenu           ;
        cp 4                            ;
        jr z,_displayPrgmMenu           ;
        cp 5                            ;
        jp nz,_JForceCmdNoChar          ;
        ld a,kMem                       ;display MEM menu
        jp _JForceKey                   ;
_displayPrgmMenu:                       ;display PRGM menu
        ld a,kPrgm                      ;
        jp _JForceKey                   ;
_displayMathMenu:                       ;display MATH menu
        ld a,kMath                      ;
        jp _JForceKey                   ;
_displayPicture:                        ;display picture
        call _clrLCD                    ;
        ld hl,bName                     ;
        rst 20h                         ;find it in memory
        rst 10h                         ;return if not found
        jp c,_JForceCmdNoChar           ;
        inc de                          ;
        inc de                          ;
        ld a,b                          ;
        ex de,hl                        ;
        call _SET_ABS_SRC_ADDR          ;
        xor a                           ;
        ld hl,$FC00                     ;
        call _SET_ABS_DEST_ADDR         ;
        ld hl,1024                      ;
        call _MM_LDIR_SET_SIZE          ;copy to video memory
        ld hl,$FFF0                     ;
_clearBottom:                           ;clear bottom line
        ld (hl),a                       ;
        inc l                           ;
        jr nz,_clearBottom              ;
        ld a,(bPicDelay)                ;delay?
        or a                            ;
        jr nz,_picDelay                 ;
_picPauseLoop:                          ;no, wait for key and exit
        call _getCSC                    ;
        or a                            ;
        jr z,_picPauseLoop              ;
        jr _picDispDone                 ;
_picDelay:                              ;yes, make delay b seconds
        ld b,a                          ;
        ei                              ;
_delayLoop1:                            ;
        push bc                         ;
        ld b,200                        ;
_delayLoop2:                            ;
        halt                            ;
        djnz _delayLoop2                ;
        pop bc                          ;
        djnz _delayLoop1                ;
_picDispDone:                           ;
        call _clrLCD                    ;exit to homescreen
        jp _JForceCmdNoChar             ;
_executeAssembly:                       ;execute assembly program
        ld hl,bName                     ;
        rst 20h                         ;
        rst 10h                         ;find it in memory
        jp c,_JForceCmdNoChar           ;return if not found
        ld hl,_errExitProgram           ;install error handler just in case
        call _pushErrorHandler          ;run it
        call _exec_assembly             ;remove error handler
        call _popErrorHandler           ;
        jp _JForceCmdNoChar             ;exit to homescreen
_executeBasic:                          ;execute basic program
        call _bufClear                  ;clear homescreen buffer
        call _PPutAwayPrompt            ;put away homescreen edit
        ld hl,bName                     ;
        rst 20h                         ;
        rst 10h                         ;find it in memory
        jp c,_JForceCmdNoChar           ;return if not found
        res onInterrupt,(iy+onFlags)    ;
        call _runIndicOn                ;turn on run indicator
        set ProgramExecuting,(iy+NewDispf)
        ld hl,_errExitProgram           ;install error handler
        call _pushErrorHandler          ;
        call _exec_basic                ;run it
        call _popErrorHandler           ;remove error handler
        call _call_cxPPutAway           ;bring back homescreen edit
        jp _JForceCmdNoChar             ;exit to homescreen
_errExitProgram:
        call _flushAllMenus             ;emergency exit to homescreen
        call _clrLCD                    ;
        call _homeUp                    ;
        jp _JForceCmdNoChar             ;

;Type: 0 = Basic program; 1 = Assembly program; 2 = Picture;
;       3 = MATH Menu; 4 = PRGM Menu; 5 = MEM Menu
bType:  .db -1
;Delay: 0 = no delay (key), 2 = 2 second delay, 5 = 5 second delay
bPicDelay:
        .db -1
;Name of variable to be executed/displayed
bName:  .db 12h,01h,"C"
.end
end
